 /*
 * Copyright 2015 The CHOReVOLUTION project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package eu.chorevolution.transformation.cdimpljaxws.utility;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PropertyConfiguration {
	
     private static Logger logger = LoggerFactory.getLogger(PropertyConfiguration.class);
     private static String PROPERTIES_FILE = "configuration.properties";
	 private static PropertyConfiguration instance = null;
	 private Map<String,Object> props = new HashMap<String,Object>();
	 
	 private PropertyConfiguration(){
		 
		 try {
	         Properties properties = new Properties();
	         properties.load(PropertyConfiguration.class.getClassLoader().getResourceAsStream(PROPERTIES_FILE));
	         for(String key : properties.stringPropertyNames()) {
	        	if(key.equals("cds_priorities")){
	        		Map<String,String> priorities = new HashMap<String,String>();
	        		List<String> prioritiesItems = Arrays.asList(properties.getProperty("cds_priorities").split(","));
	        		for (String prioritiesItem : prioritiesItems) {
	        			List<String> item = Arrays.asList(prioritiesItem.split(":"));
	        			priorities.put(item.get(0),item.get(1));
					}
	        		props.put("cds_priorities",priorities);
	        	}
	        	else{
		            props.put(key, properties.getProperty(key));      		
	        	}
	         }
	     }catch (IOException e) {
	    	 logger.info(e.getMessage());
	     }
	 }
	 
	 public static synchronized PropertyConfiguration getInstance(){
	      
		 if (instance == null)
	           instance = new PropertyConfiguration();
	     return instance;
	 }

	 public String getStringValue(String key){
	 
		 return (String) props.get(key);
	 }
	 
	 public Map<String,String> getMapValue(String key){
		 
		 return (Map<String, String>) props.get(key);
	 }
	 
	 

}
